# -*- coding: utf-8 -*-
"""
Created on Fri Aug  5 14:05:41 2022

@author: Petr Tomasek
"""

# Solver for FDE Backward Euler method
# y'(x)=f(x,y(x)),  y(xinit)=yinit

#import fnGraphs as fnG
import numpy as np
import math
import fnNumMethCaputoFDE as FnEM
import matplotlib
 
#matplotlib.use('Agg') # musi byt pred importem pyplot
matplotlib.use('PDF') # musi byt pred importem pyplot
 
import matplotlib.pyplot as graph
from matplotlib.font_manager import FontProperties


font = FontProperties()
font.set_style('italic')
font.size=20

# INPUT
def f(t,y): #RHS of FDE   
    #return(9*pow(2*y-2, 2/3)*t/(10*math.gamma(2/3)));
    return(9*pow(t, 5/3)/(10*math.gamma(2/3)));

alpha=1/3; #fractional derivative order     
tinit=0;
yinit=1;
tfinal=10;
h=0.1 #stepsize
IC=[tinit,yinit]
t=np.arange(IC[0],tfinal+h,h) # Mesh initialization
FE_y=np.zeros((3,len(t)));

# Computing process and graphical output
graph.figure(figsize=(10,6), dpi=120)
matplotlib.rcParams['font.size']=20
graph.grid(True)
graph.xlabel("t",fontproperties=font)
graph.ylabel("y",rotation=0, labelpad=15,fontproperties=font)
pr=["k-o","k-.s","k--x","k-.p","r-o"]
# Fractional forward Euler method
FE_y[0]=FnEM.fnFrwEulMethod(f,alpha,tinit,yinit,tfinal,IC,h)
# Fractional backward Euler method
FE_y[1]=FnEM.fnBkwEulMethod(f,alpha,tinit,yinit,tfinal,IC,h)
# Analytical solution of the task
FE_y[2]=pow(t,2)/2+1;

   
graph.figure(figsize=(10,6), dpi=120)
matplotlib.rcParams['font.size']=20
graph.grid(True)
#graph.title("Solution to FDE IVP",fontsize=20)
graph.xlabel("t",fontproperties=font)
graph.ylabel("y",rotation=0, labelpad=15,fontproperties=font)
graph.plot(t, FE_y[0,:], pr[0],label = r"FFEM")
graph.plot(t, FE_y[1,:], pr[1],label = r"FBEM")
graph.plot(t, FE_y[2,:], pr[2],label = r"$t^2/2+1$")
graph.legend(loc = 0)
graph.show()
graph.savefig("E2plot.pdf", dpi=120);   



#Absolute error table in given values of t:
    # the value of r should be modified with respect to chosen value of h
for k in range(0,11):
   # r=10 for h=0.1
   # r=100 for h=0.01
   # r=1000 for h=0.001
   r=10*k 
   # print(t[r],FE_y[0,r],FE_y[1,r],FE_y[2,r])
   print("FFEM: Absolute error in t=",t[r],"=",abs(FE_y[0,r]-FE_y[2,r]))
   #print("FBEM: Absolute error in t=",t[r],"=",abs(FE_y[1,r]-FE_y[2,r]))   

